# Copyright 2013-2019 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from spack import *


class BuildahTest(MakefilePackage):
    """The Buildah package provides a command line tool that can be used to

    * create a working container, either from scratch or using an image as a starting point
    * create an image, either from a working container or via the instructions in a Dockerfile
    * images can be built in either the OCI image format or the traditional upstream docker image format
    * mount a working container's root filesystem for manipulation
    * unmount a working container's root filesystem
    * use the updated contents of a container's root filesystem as a filesystem layer to create a new image
    * delete a working container or an image
    * rename a local container
"""

    homepage = "https://buildah.io/"
    url      = "https://github.com/containers/buildah/archive/v1.14.5.tar.gz"

    version('1.29.1', sha256='ac20cdbd81616f3f03ef42dc3139de118e3b6d0ed737c994c45dc1039e0fa1c1')
    version('1.15.0', sha256='ffa3b84f3ef1cc482a3e9e27e8f7417e4dd5c70bee7c6208155b9e2039b1d606')
    version('1.14.9', sha256='fa084af683a9f91bdc0fe24077ad56ec1bc9f4f98445ddcba4e0796710281adf')
    version('1.14.5', sha256='74633280c814d340ed32653106bc706f306cf78afb57bb51c3cdca8893d95bd4')
    version('1.14.0', sha256='3eb637c5a5b35ba865886d20ce937e7c6b2929573c03af09a86c3be11151ec80')
   
    depends_on('bzip2')
    depends_on('cni-plugins', type='run')
    depends_on('go',type='build')
    depends_on('go-md2man', type='build')
    depends_on('gpgme')
    depends_on('libassuan')
    depends_on('libgpg-error')
    depends_on('lvm2')
    depends_on('libseccomp')
    depends_on('runc@1.0:', type='run')

    # ALCF config
    depends_on("fuse-overlayfs", type="run")

    def patch(self):
        # Add runtime executables to default search paths
        defs = FileFilter("vendor/github.com/containers/common/pkg/config/default.go")

        # Prepend the provided runc executable to podman's built-in runc search path
        defs.filter('"runc": {', '"runc": {' + '"{0}",'.format(self.spec["runc"].prefix.sbin.runc))
        # Prepend the provided cni-plugins directory to the cni-plugin search path
        defs.filter(
            r"DefaultCNIPluginDirs = \[\]string{",
            "DefaultCNIPluginDirs = []string{"
            + '\n        "{0}",'.format(self.spec["cni-plugins"].prefix.bin),
        )

        usr_prefix_patching = [
            "common/pkg/config/config.go",
            "common/pkg/config/containers.conf",   
            "common/pkg/config/default.go",   
            "common/pkg/subscriptions/subscriptions.go",
            "common/pkg/hooks/hooks.go",
            "storage/types/options_linux.go",
            "storage/types/options_darwin.go",
        ]

        etc_prefix_patching = [
            "common/libnetwork/network/interface_linux.go",
            "common/libnetwork/network/interface.go",
            "common/pkg/config/config.go",
            "common/pkg/config/config_linux.go",
            "common/pkg/config/containers.conf",
            "common/pkg/config/config_darwin.go",
            "common/pkg/subscriptions/subscriptions.go",
            "common/pkg/machine/machine.go",
            "common/pkg/hooks/hooks.go",
            "image/v5/docker/paths_common.go",
            "storage/types/options_linux.go",
            "storage/types/options_darwin.go",
            "storage/storage.conf",
        ]

        # Filter storage config options
        for f in usr_prefix_patching:
            fullpath = "vendor/github.com/containers/" + f
            defs = FileFilter(fullpath)
            defs.filter(
                r"/usr/share/",
                "{0}/share/".format(self.prefix)
            )

        for f in etc_prefix_patching:
            fullpath = "vendor/github.com/containers/" + f
            defs = FileFilter(fullpath)
            defs.filter(
                r"/etc/",
                "{0}/etc/".format(self.prefix)
            )

    def edit(self, spec, prefix):
        env['EXTRA_LDFLAGS'] = (
            "-X github.com/containers/image/v5/sysregistries.systemRegistriesConfPath={0}/etc/containers/registries.conf"
            " -X github.com/containers/image/v5/sysregistries.systemRegistriesConfDirPath={0}/etc/containers/registries.conf.d"
            " -X github.com/containers/image/v5/signature.systemDefaultPolicyPath={0}/etc/containers/policy.json"
            " -X github.com/containers/image/v5/docker.systemRegistriesDirPath={0}/etc/containers/registries.d"
            " -X github.com/containers/common._etcDir={0}/etc"
            " -X github.com/containers/common._installPrefix={0}"
        ).format(self.prefix)

    def build(self, spec, prefix):
        make()

    def install(self, spec, prefix):
        make( 'install',
             'PREFIX={0}'.format(prefix))
