# Copyright 2013-2023 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)
import os

import llnl.util.filesystem as fs
import llnl.util.tty as tty

from spack.build_systems import autotools, nmake
from spack.package import *


class Libxml2(AutotoolsPackage, NMakePackage):
    """Libxml2 is the XML C parser and toolkit developed for the Gnome
    project (but usable outside of the Gnome platform), it is free
    software available under the MIT License."""

    homepage = "http://xmlsoft.org"
    url = "https://download.gnome.org/sources/libxml2/2.9/libxml2-2.9.13.tar.xz"
    list_url = "https://gitlab.gnome.org/GNOME/libxml2/-/releases"

    def url_for_version(self, version):
        url = "https://download.gnome.org/sources/libxml2/{0}/libxml2-{1}.tar.xz"
        return url.format(version.up_to(2), version)

    version("2.10.3", sha256="5d2cc3d78bec3dbe212a9d7fa629ada25a7da928af432c93060ff5c17ee28a9c")
    version("2.10.2", sha256="d240abe6da9c65cb1900dd9bf3a3501ccf88b3c2a1cb98317d03f272dda5b265")
    version("2.10.1", sha256="21a9e13cc7c4717a6c36268d0924f92c3f67a1ece6b7ff9d588958a6db9fb9d8")
    version("2.9.14", sha256="60d74a257d1ccec0475e749cba2f21559e48139efba6ff28224357c7c798dfee")
    version("2.9.13", sha256="276130602d12fe484ecc03447ee5e759d0465558fbc9d6bd144e3745306ebf0e")
    version("2.9.12", sha256="28a92f6ab1f311acf5e478564c49088ef0ac77090d9c719bbc5d518f1fe62eb9")
    version("2.9.11", sha256="a4a59441c8dc1dd82ca6857fa2d640aff2e1ab65ee480caa0d30e3626167a204")
    version("2.9.10", sha256="593b7b751dd18c2d6abcd0c4bcb29efc203d0b4373a6df98e3a455ea74ae2813")
    version("2.9.9", sha256="58a5c05a2951f8b47656b676ce1017921a29f6b1419c45e3baed0d6435ba03f5")
    version("2.9.8", sha256="dcca21d624bbbe094fcc104e1f15f2eacfb65aecd0e38ed220aeca56b62c81e2")
    version("2.9.7", sha256="6437855f3332fcc0fc25323c5f06901e02277e1232a662b621dc88fde37259ac")
    version("2.9.4", sha256="21b2824f9ce56a72296f678103eed5a8446417c07e904e61748affa230ec9072")
    version("2.9.2", sha256="edd7a2fed2eb832a094992aec61f1c41167a095dd8ce7f3e29db42c539eb4ff2")
    version("2.7.8", sha256="7d7e03ba5f1d985d15f75e04c674babcb18e70a4d41cc8686523fdd21f43ce36")

    variant("python", default=False, description="Enable Python support")

    depends_on("pkgconfig@0.9.0:", type="build", when="build_system=autotools")
    # conditional on non Windows, but rather than specify for each platform
    # specify for non Windows builder, which has equivalent effect
    depends_on("iconv", when="build_system=autotools")
    depends_on("zlib")
    depends_on("xz")

    # avoid cycle dependency for concretizer
    with when("+python"):
        extends("python")
        depends_on("python+shared~libxml2")
        # A note about python versions: libxml 2.10.1 (and presumably earlier) has
        # a bug in its configure script that fails to properly parse python
        # version strings with more than one character for the minor version.
        depends_on("python@:3.9", when="@:2.10.1")

    # XML Conformance Test Suites
    # See https://www.w3.org/XML/Test/ for information
    resource(
        name="xmlts",
        url="https://www.w3.org/XML/Test/xmlts20080827.tar.gz",
        sha256="96151685cec997e1f9f3387e3626d61e6284d4d6e66e0e440c209286c03e9cc7",
    )

    patch("nvhpc-elfgcchack.patch", when="@:2.9 %nvhpc")

    # Use NAN/INFINITY if available to avoid SIGFPE
    # See https://gitlab.gnome.org/GNOME/libxml2/-/merge_requests/186
    patch(
        "https://gitlab.gnome.org/GNOME/libxml2/-/commit/c9925454fd384a17c8c03d358c6778a552e9287b.patch",
        sha256="3e06d42596b105839648070a5921157fe284b932289ffdbfa304ddc3457e5637",
        when="@2.9.11:2.9.14",
    )
    build_system(conditional("nmake", when="platform=windows"), "autotools", default="autotools")

    @property
    def command(self):
        return Executable(self.prefix.bin.join("xml2-config"))

    @property
    def headers(self):
        include_dir = self.spec.prefix.include.libxml2
        hl = find_all_headers(include_dir)
        hl.directories = [include_dir, self.spec.prefix.include]
        return hl

    def patch(self):
        # Remove flags not recognized by the NVIDIA compiler
        if self.spec.satisfies("%nvhpc"):
            filter_file(
                "-pedantic -Wall -Wextra -Wshadow -Wpointer-arith "
                "-Wcast-align -Wwrite-strings -Waggregate-return "
                "-Wstrict-prototypes -Wmissing-prototypes "
                "-Wnested-externs -Winline -Wredundant-decls",
                "-Wall",
                "configure",
            )
            filter_file("-Wno-long-long -Wno-format-extra-args", "", "configure")

    def test(self):
        """Perform smoke tests on the installed package"""
        # Start with what we already have post-install
        tty.msg("test: Performing simple import test")
        self.import_module_test()

        data_dir = self.test_suite.current_test_data_dir

        # Now run defined tests based on expected executables
        dtd_path = data_dir.join("info.dtd")
        test_filename = "test.xml"
        exec_checks = {
            "xml2-config": [("--version", [str(self.spec.version)], 0)],
            "xmllint": [
                (["--auto", "-o", test_filename], [], 0),
                (
                    ["--postvalid", test_filename],
                    ["validity error", "no DTD found", "does not validate"],
                    3,
                ),
                (
                    ["--dtdvalid", dtd_path, test_filename],
                    ["validity error", "does not follow the DTD"],
                    3,
                ),
                (["--dtdvalid", dtd_path, data_dir.join("info.xml")], [], 0),
            ],
            "xmlcatalog": [("--create", ["<catalog xmlns", 'catalog"/>'], 0)],
        }
        for exe in exec_checks:
            for options, expected, status in exec_checks[exe]:
                self.run_test(exe, options, expected, status)

        # Perform some cleanup
        fs.force_remove(test_filename)


class RunAfter(object):
    @run_after("install")
    @on_package_attributes(run_tests=True)
    def import_module_test(self):
        if "+python" in self.spec:
            with working_dir("spack-test", create=True):
                python("-c", "import libxml2")


class AutotoolsBuilder(autotools.AutotoolsBuilder, RunAfter):
    def configure_args(self):
        spec = self.spec

        args = [
            "--with-lzma={0}".format(spec["xz"].prefix),
            "--with-iconv={0}".format(spec["iconv"].prefix),
        ]

        if "+python" in spec:
            args.extend(
                [
                    "--with-python={0}".format(spec["python"].home),
                    "--with-python-install-dir={0}".format(python_platlib),
                ]
            )
        else:
            args.append("--without-python")

        return args


class NMakeBuilder(nmake.NMakeBuilder, RunAfter):
    phases = ("configure", "build", "install")

    @property
    def makefile_name(self):
        return "Makefile.msvc"

    @property
    def build_directory(self):
        return os.path.join(self.stage.source_path, "win32")

    def configure(self, pkg, spec, prefix):
        with working_dir(self.build_directory):
            opts = [
                "prefix=%s" % prefix,
                "compiler=msvc",
                "iconv=no",
                "zlib=yes",
                "lzma=yes",
                "lib=%s" % ";".join((spec["zlib"].prefix.lib, spec["xz"].prefix.lib)),
                "include=%s" % ";".join((spec["zlib"].prefix.include, spec["xz"].prefix.include)),
            ]
            if "+python" in spec:
                opts.append("python=yes")
            cscript("configure.js", *opts)
